#include <i2c_8x2lcd_st7032.h>
#include <Wire.h>


I2C_8X2LCD::I2C_8X2LCD(){
}

I2C_8X2LCD::I2C_8X2LCD(int vccpin,int gndpin){
  pinMode(gndpin, OUTPUT);
  digitalWrite(gndpin, LOW);
  pinMode(vccpin, OUTPUT);
  digitalWrite(vccpin, HIGH);
}

void I2C_8X2LCD::init(int contrast){
  delay(500);
  Wire.begin();
  lcd_cmd(0b00111000); // function set
  lcd_cmd(0b00111001); // function set
  lcd_cmd(0b00000100); // EntryModeSet
  lcd_cmd(0b00010100); // interval osc
  lcd_cmd(0b01110000 | (contrast & 0xF)); // contrast Low
  lcd_cmd(0b01011100 | ((contrast >> 4) & 0x3)); // contast High/icon/power
  lcd_cmd(0b01101100); // follower control
  delay(200);
  lcd_cmd(0b00111000); // function set
  lcd_cmd(0b00001100); // Display On
  lcd_cmd(0b00000001); // Clear Display
  delay(2);
}

void I2C_8X2LCD::lcd_cmd(byte x) {
  Wire.beginTransmission(I2Cadr);
  Wire.write(0b00000000); // CO = 0,RS = 0
  Wire.write(x);
  Wire.endTransmission();
  delayMicroseconds(27);    // >26.3us
}

void I2C_8X2LCD::cursor(bool enable,bool position) {
  lcd_cmd(0b00001100|(position?0b00000010:0)|(enable?0b00000001:0));
}

void I2C_8X2LCD::cursor_move(bool right) {
  lcd_cmd(0b00010000|(right?0b00000100:0));
}

void I2C_8X2LCD::screen_move(bool right) {
  lcd_cmd(0b00011000|(right?0b00000100:0));
}
  
void I2C_8X2LCD::contdata(byte x) {
  Wire.write(0b11000000); // CO = 1, RS = 1
  Wire.write(x);
}
 
void I2C_8X2LCD::lastdata(byte x) {
  Wire.write(0b01000000); // CO = 0, RS = 1
  Wire.write(x);
}
 
void I2C_8X2LCD::printStr(byte x,byte y,const char *s){
	setCursor(x,y);
	printStr(s);
}
 
// ̕\
void I2C_8X2LCD::printStr(const char *s) {
  Wire.beginTransmission(I2Cadr);
  while (*s) {
    if (*s == -17){ //pJĩwb_
      s++;
      if (*s == -66){
        s++;
        if (*(s + 1)) {
          contdata(*s+0x40);
        } else {
          lastdata(*s+0x40);
        }
      }	 
    } else if (*(s + 1)) {
      contdata(*s);
    } else {
      lastdata(*s);
    }
    s++;
  }
  Wire.endTransmission();
}

void I2C_8X2LCD::printCh(byte x,byte y,const char s){
	setCursor(x,y);
	printCh(s);
}
 
void I2C_8X2LCD::printCh(const char s) {
  Wire.beginTransmission(I2Cadr);
  lastdata(s);
  Wire.endTransmission();
}

void I2C_8X2LCD::printInt(int num)
{
  char int2str[10]; 
  sprintf(int2str,"%d",num);
  printStr(int2str);	
}

// \ʒu̎w
void I2C_8X2LCD::setCursor(byte x, byte y) {
  lcd_cmd(0x80 | (y * 0x40 + x));
}

void I2C_8X2LCD::setContrast(byte c) {

  lcd_cmd(0x39);
  lcd_cmd(0b01110000 | (c & 0x0f)); // contrast Low
  lcd_cmd(0b01011100 | ((c >> 4) & 0x03)); // contast High/icon/power
  lcd_cmd(0x38);
}

void I2C_8X2LCD::clear(){
  lcd_cmd(0b00000001); // Clear Display
  delay(2000);
}

void I2C_8X2LCD::doublemode(bool enable){
  lcd_cmd(enable?0b00110100:0b00111000);
}

void I2C_8X2LCD::customChar(int index, uint8_t charmap[]){
  lcd_cmd(0b01000000|((index&0x07)<<3));
  for (int i=0;i<8;i++){
    Wire.beginTransmission(I2Cadr);
  	Wire.write(0b01000000);
  	Wire.write(charmap[i]);
  	Wire.endTransmission();
  	delayMicroseconds(27);    // >26.3us
  }
}

